<?php
require_once(dirname(__FILE__).DIRECTORY_SEPARATOR.'SprypayCurl.php');
require_once(dirname(__FILE__).DIRECTORY_SEPARATOR.'SprypayException.php');
require_once(dirname(__FILE__).DIRECTORY_SEPARATOR.'SprypayApiException.php');
/**
 * Класс для работы с API платежной системы Sprypay.
 * 
 * @package Sprypaylib
 * @version $id$
 * @copyright 2012 by Sprypay {@link http://sprypay.ru}
 * @license BSD
 */
 class SprypayApi
 {
     const SUCCESS_CODE = 1;

     private $email;
     private $password;
     private $curl;
     private $apiUrl = 'http://sprypay.ru/api/api.php';
     private $requestMethod = 'get';

     /**
      * Устанавливает email пользователя Sprypay.
      * 
      * @param string $email 
      * @access public
      * @return void
      */
     public function setUserEmail($email)
     {
         $this->email = $email;
     }

     /**
      * Возвращает email пользователя Sprypay. 
      * 
      * @access public
      * @return string
      */
     public function getUserEmail()
     {
         return $this->userEmail;
     }

     /**
      * Устанавливает пароль к аккаунту пользователя в Sprypay.
      * 
      * @param string $password
      * @access public
      * @return void
      */
     public function setUserPassword($password)
     {
         $this->password = $password;
     }

     /**
      * Возвращает пароль к аккаунту пользователя в Sprypay.
      * 
      * @access public
      * @return string
      */
     public function getUserPassword()
     {
         return $this->password;
     }

     /**
      * Устанавливает URL для API Sprypay.
      * 
      * @param string $url
      * @access public
      * @return void
      */
     public function setApiUrl($url)
     {
         $this->apiUrl = $url;
         if ($this->getRequestMethod() == 'post')
             $this->curl->setOption(CURLOPT_URL, $url);
     }

     /**
      * Возвращает URL для API Sprypay.
      * 
      * @access public
      * @return string
      */
     public function getApiUrl()
     {
         return $this->apiUrl;
     }

     /**
      * Устанавливает метод при помощи которого будут осуществляться запросы.
      * 
      * @param string $method "post" | "get"
      * @access public
      * @return void
      */
     public function setRequestMethod($method)
     {
         if ($method == 'get') {
             $this->requestMethod = 'get';
             $this->curl->setOption(CURLOPT_HTTPGET, true);
         } elseif ($method == 'post') {
             $this->requestMethod = 'post';
             $this->curl->setOption(CURLOPT_URL, $this->apiUrl);
             $this->curl->setOption(CURLOPT_POST, true);
         } else
             throw new SprypayApiException('Unknown request method '.$method);
     }

     /**
      * Возвращает метод при помощи которого будут осуществляться запросы.
      * 
      * @access public
      * @return string
      */
     public function getRequestMethod()
     {
         return $this->requestMethod;
     }

     /**
      * Осуществляет запрос к API Sprypay.
      * 
      * @param string $action имя действия
      * @param array $params параметры запроса
      * @access public
      * @return mixed результат выполнения запроса
      */
     public function request($action, array $params = array())
     {
         $params = $this->prepareRequestParams($action, $params);
         if ($this->requestMethod == 'get') {
             $result = $this->requestByGet($params);
         } else {
             $result = $this->requestByPost($params);
         }
         return $this->handleResponse($action, $result);
     }

     private function requestByGet($params)
     {
         $url = $this->apiUrl.'?'.$params;
         $this->curl->setOption(CURLOPT_URL, $url);
         return $this->curl->execute();
     }

     private function requestByPost($params)
     {
         $this->curl->setOption(CURLOPT_POSTFIELDS, $params);
         return $this->curl->execute();
     }

     private function prepareRequestParams($action, array $params = array())
     {
         $params['action']     = $action;
         $params['userEmail']  = $this->email;
         $params['userPasswd'] = md5($this->password);
         $preparedParams = '';
         foreach ($params as $key => $param)
             $preparedParams .= $key.'='.urlencode($param).'&';
        $preparedParams = substr($preparedParams, 0, -1);
        return $preparedParams;
     }

     private function handleResponse($action, $result)
     {
         $xml = @simplexml_load_string($result);
         if (!$xml)
             throw new SprypayApiException('Error when parsing response xml');

         $resultCode = $this->parseResponseResultCode($xml);
         if ($resultCode != self::SUCCESS_CODE) {
             $errorMessage = $this->parseResponseErrorMessage($xml);
             throw new SprypayApiException($errorMessage, $resultCode);
         }

         $handler = 'handle'.ucfirst($action);
         if (method_exists($this, $handler)) {
             $responseBody = $this->parseResponseBody($xml);
             return call_user_func(array($this, $handler), $responseBody);
         } else
             return $result;
     }

     private function parseResponseResultCode($xml)
     {
         if (isset($xml->resultCode))
             return (int) $xml->resultCode;
         else
             throw new SprypayApiException('Result code not found in response');
     }

     private function parseResponseBody($xml)
     {
         if (isset($xml->responseBody))
             return $xml->responseBody;
         else
             throw new SprypayApiException('Response body not found in response');
     }

     private function parseResponseErrorMessage($xml)
     {
         if (isset($xml->responseBody->errorMessage))
             return (string) $xml->responseBody->errorMessage;
         else
             return 'Unknown error';
     }

     private function handleGetPaymentSystems($data)
     {
         $paymentSystems = array();
         foreach ($data->psData as $paymentSystem) {
             $ps = new stdClass;
             foreach ($paymentSystem as $key => $value)
                 $ps->$key = (string) $value;
             $paymentSystems[$ps->paymentSystemId] = $ps;
         }
         return $paymentSystems;
     }

     private function handleGetShopPaymentSystems($data)
     {
         return $this->handleGetPaymentSystems($data);
     }

     private function handleGetPaymentSystemsTypes($data)
     {
         $paymentSystemsTypes = array();
         foreach ($data->type as $paymentSystemType) {
             $t = new stdClass;
             foreach ($paymentSystemType as $key => $value)
                 $t->$key = (string) $value;
             $paymentSystemsTypes[$t->typeId] = $t;
         }
         return $paymentSystemsTypes;
     }

     private function handleGetBalance($data)
     {
         $balances = array();
         foreach ($data->children() as $currency => $balance) {
             $b = new stdClass;
             $b->currencyCode = $currency;
             foreach ($balance as $key => $value)
                 $b->$key = (string) $value;
            $balances[$currency] = $b;
         }
         return $balances;
     }

     private function handleCheckInternalTransfer($data)
     {
         $transfer = new stdClass;
         foreach ($data->children() as $key => $value)
             $transfer->$key = (string) $value;
         return $transfer;
     }

     private function handleMakeInternalTransfer($data)
     {
         return $this->handleCheckInternalTransfer($data);
     }

     /**
      * Конструктор класса.
      * 
      * @param array $config параметры конфигурации 
      * @access public
      * @return void
      */
     public function __construct(array $config = array())
     {
        if (!extension_loaded('simplexml'))
            throw new SprypayApiException('SimpleXML extension not found');

         $curlOptions = array();
         if (isset($config['curl_options']) && is_array($config['curl_options']))
             $curlOptions = $config['curl_options'];
         $this->initCurl($curlOptions);

         if (!empty($config)) {
             if (isset($config['email']))
                 $this->setUserEmail($config['email']);
             if (isset($config['password']))
                 $this->setUserPassword($config['password']);
             if (isset($config['api_url']))
                 $this->setApiUrl($config['api_url']);
             if (isset($config['request_method']))
                 $this->setRequestMethod($config['request_method']);
         }
     }

     private function initCurl(array $curlOptions = array())
     {
         $this->curl = new SprypayCurl();
         if (!empty($curlOptions))
             $this->curl->setOptions($curlOptions);
         $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
     }
 }
 ?>
